import cv2
import numpy as np
import socket
import sys
import pickle
import struct
import time
import logging
import os

logging.basicConfig(level=logging.INFO)

HOST = '192.168.1.26'
PORT = 8100

def initialise_data():
    global HOST, PORT
    if os.environ.get('HOST_CLIENT'):
        HOST = os.environ.get('HOST_CLIENT')
    if os.environ.get('PORT_CLIENT'):
        PORT = int(os.environ.get('PORT_CLIENT'))

def initialise_client(host, port):
    connecting = True

    logging.info('Capturing video')

    cap=cv2.VideoCapture(0)
    width = 480
    height = 320
    cap.set(cv2.CAP_PROP_FRAME_WIDTH, width)
    cap.set(cv2.CAP_PROP_FRAME_HEIGHT, height)
    clientsocket=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    
    while connecting:
        try:
            logging.info('Connecting...')
            clientsocket.connect((host,port))
            connecting = False
            logging.info('Connection Succesful!')
        except:
            logging.info('Cannot establish connection. Retrying in 3 seconds...')
            time.sleep(3)

    while True:
        if cap.isOpened():
            ret,frame=cap.read()
            grey = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
            data = pickle.dumps(grey)

            message_size = struct.pack("L", len(data))

            clientsocket.sendall(message_size + data)
        else:
            logging.info('Camera not open')


if __name__ == '__main__':
    initialise_data()
    initialise_client(HOST, PORT)
