import cv2
import numpy as np
import socket
import sys
import pickle
import struct
import time
import threading
import logging
import os

logging.basicConfig(level=logging.INFO)

circle_frames = None

HOST_SERVER = '192.168.1.21'
PORT_SERVER = int('8099')

HOST_CLIENT = '192.168.1.21'
PORT_CLIENT = int('8100')

def initialise_data():
    global HOST_SERVER, PORT_SERVER, HOST_CLIENT, PORT_CLIENT
    if os.environ.get('HOST_SERVER'):
        HOST_SERVER = os.environ.get('HOST_SERVER')
    if os.environ.get('PORT_SERVER'):
        PORT_SERVER = int(os.environ.get('PORT_SERVER'))
    if os.environ.get('HOST_CLIENT'):
        HOST_CLIENT = os.environ.get('HOST_CLIENT')
    if os.environ.get('PORT_CLIENT'):
        PORT_CLIENT = int(os.environ.get('PORT_CLIENT'))

def initialise_client(host_client, port_client):
    connecting = True
    global circle_frames

    clientsocket=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    while connecting:
        try:
            logging.info('Connecting...')
            clientsocket.connect((host,port))
            connecting = False
            logging.info('Connection Succesful!')
        except:
            logging.info('Cannot establish connection. Retrying in 3 seconds...')
            time.sleep(3)

    while True:
        data = pickle.dumps(circle_frames)
        message_size = struct.pack("L", len(data))
        clientsocket.sendall(message_size + data)

def initialise_server(host_server, port_server):
    logging.info(host_server)
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    logging.info('Socket created')

    s.bind((host_server, port_server))
    logging.info('Socket bind complete')
    s.listen(10)
    logging.info('Socket now listening')

    conn, addr = s.accept()

    data = b''
    payload_size = struct.calcsize("L")

    global circle_frames

    while True:
        while len(data) < payload_size:
            data += conn.recv(4096)

        packed_msg_size = data[:payload_size]
        data = data[payload_size:]
        msg_size = struct.unpack("L", packed_msg_size)[0]

        while len(data) < msg_size:
            data += conn.recv(4096)

        frame_data = data[:msg_size]
        data = data[msg_size:]

        circle_frames = pickle.loads(frame_data)

        sc = 1
        md = 20
        at = 50

        circles = cv2.HoughCircles(circle_frames, cv2.HOUGH_GRADIENT, sc, md, param1=65,param2=35,minRadius=0,maxRadius=0)

        if circles is not None:
            circle = circles[0][0]
            if circle.size > 1:
                x, y, radius = int(circle[0]), int(circle[1]), int(circle[2])
                cv2.circle(circle_frames, (x, y), radius, (0, 0, 255), 2)
                cv2.circle(circle_frames, (x, y), 1, (0, 0, 255), 2)

if __name__ == '__main__':
    initialise_data()

    t1 = threading.Thread(target=initialise_server, args=(HOST_SERVER, PORT_SERVER))
    t1.start()

    t2 = threading.Thread(target=initialise_client, args=(HOST_CLIENT, PORT_CLIENT))
    t2.start()
