import cv2
import pickle
import socket
import struct
import logging
import threading
import http.server
import socketserver
import os

logging.basicConfig(level=logging.INFO)

HOST_SERVER = '0.0.0.0'
PORT_SERVER = 8101
jpg = None

def initialise_data():
    global HOST_SERVER, PORT_SERVER
    if os.environ.get('HOST_SERVER'):
        HOST_SERVER = os.environ.get('HOST_SERVER')
    if os.environ.get('PORT_SERVER'):
        PORT_SERVER = int(os.environ.get('PORT_SERVER'))

pageData = "<!DOCTYPE>" + \
            "<html>" + \
            "  <head>" + \
            "    <title>HttpListener Example</title>" + \
            "  </head>" + \
            "  <body>" + \
            "<img id=\"image\"/>"+ \
            " <script type=\"text/javascript\">var image = document.getElementById('image');function refresh() {image.src = \"/image?\" + new Date().getTime();image.onload= function(){setTimeout(refresh, 30);}}refresh();</script>   "+ \
            "  </body>" + \
            "</html>"

class MyHandler(http.server.BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path == '/':
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.end_headers()
            self.wfile.write(bytes(pageData, "utf8"))
        elif self.path.startswith('/image'):
            self.send_response(200)
            self.send_header("Content-type", "image/jpeg")
            self.end_headers()
            
            global jpg
            if jpg is not None:
                self.wfile.write(jpg)
        else:
            self.send_response(404)


def initialise_server(host, port):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    logging.info('Socket created')

    s.bind((host, port))
    logging.info('Socket bind complete')
    s.listen(10)
    logging.info('Socket now listening')

    conn, addr = s.accept()

    data = b''
    payload_size = struct.calcsize("L")

    while True:
        while len(data) < payload_size:
            data += conn.recv(4096)

        packed_msg_size = data[:payload_size]
        data = data[payload_size:]
        msg_size = struct.unpack("L", packed_msg_size)[0]

        while len(data) < msg_size:
            data += conn.recv(4096)
        frame_data = data[:msg_size]
        data = data[msg_size:]

        global jpg
        frame = pickle.loads(frame_data)
        _, jpg = cv2.imencode(".jpg", frame)

def initialise_browser():
    with socketserver.TCPServer((HOST_SERVER, PORT_SERVER+1), MyHandler) as httpd:
        print("Serving at port ", PORT_SERVER+1)
        try:
            httpd.serve_forever()
        except:
            pass

if __name__ == '__main__':
    initialise_data()

    t1 = threading.Thread(target=initialise_server, args=(HOST_SERVER, PORT_SERVER))
    t1.start()

    t2 = threading.Thread(target=initialise_browser)
    t2.start()
